# -*- coding: utf-8 -*-
"""
The MIT License (MIT)

Copyright 2015 Umbrella Tech.

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
from __future__ import unicode_literals
from django.db import models
from django.db.models import CharField, DateField
from python_brfied import UF_LIST
from python_brfied import ZonaChoices
from django_brfied.django_brfied.models import CEPField, UFField, MunicipioField


__author__ = 'Kelson da Costa Medeiros <kelsoncm@gmail.com>'


class EnderecoMixin(models.Model):
    endereco_logradouro = CharField('Logradouro', max_length=150, null=False, blank=False)
    endereco_numero = CharField('Número', max_length=150, null=False, blank=False)
    endereco_complemento = CharField('Complemento', max_length=150, null=True, blank=True)
    endereco_bairro = CharField('Bairro', max_length=150, null=False, blank=False)
    endereco_municipio = MunicipioField(null=False, blank=False)
    endereco_cep = CEPField(null=False, blank=False)
    endereco_referencia = CharField('Referência', max_length=150, null=False, blank=False)
    endereco_zona = CharField('Zona residencial', max_length=150, choices=ZonaChoices.CHOICES, null=False, blank=False)

    class Meta:
        abstract = True

    @property
    def endereco(self):
        return '%s, %s - %s, %s, %s' % \
               (self.endereco_logradouro, self.endereco_numero, self.endereco_complemento, self.endereco_bairro,
                self.endereco_municipio, )

    @property
    def enderecamento_correios(self):
        return '%s %s %s\n%s\n%s\n%s' % \
               (self.endereco_logradouro, self.endereco_numero, self.endereco_complemento, self.endereco_bairro,
                self.endereco_municipio, self.endereco_cep,)


class RegistroGeralMixin(models.Model):
    rg_numero = CharField('Número do RG', max_length=20, null=False, blank=False)
    rg_orgao = CharField('Órgão expedidor do RG', max_length=20, null=False, blank=False)
    rg_uf = UFField('UF de expedição do RG', max_length=2, choices=UF_LIST, null=False, blank=False)

    class Meta:
        abstract = True

    @property
    def rg(self):
        return '%s %s/%s' % (self.rg_numero, self.rg_orgao, self.rg_uf)


class NascimentoMixin(models.Model):
    nascimento_data = DateField('Data de nascimento', null=False, blank=False)
    nascimento_municipio = MunicipioField('Município de nascimento', max_length=150, null=False, blank=False)
    nascimento_pais = CharField('País de nascimento', max_length=150, null=False, blank=False)

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-
"""
BR-specific Form helpers


The MIT License (MIT)

Copyright 2015 Umbrella Tech.

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
from __future__ import unicode_literals
from django.forms import ValidationError
from django.forms.fields import CharField
from python_brfied import validate_dv_by_mask, validate_mask, validate_mod11, validate_cnpj
from python_brfied import only_digits, apply_mask
from python_brfied import CPF_MASK, CNPJ_MASK, CEP_MASK
from python_brfied import ValidationException

__all__ = ['MaskField', 'CPFField', 'CNPJField', 'CEPField', ]

__author__ = 'Kelson da Costa Medeiros <kelsoncm@gmail.com>'


class MaskField(CharField):
    """
    Um campo de formulário que valida um número mascarado genérico (CPF, CNPJ, Processo, Agencia, Conta-Corrente, ).
    Informe a máscara no argumento mask. Obrigatório.
    Se desejar obrigar que o valor venha com máscara informe mask_required=True. Opcional, default = False.
    Se desejar forçar a máscara para valores imcompletos informe mask_forced=True. Opcional, default = True.
    Ainda não suporta máscaras multiplas (CNPF).
    Ainda não suporta escolher o módulo.
    """

    def __init__(self, mask, mask_forced=True, mask_required=False, *args, **kwargs):
        if mask_forced and mask_required:
            raise ValidationException('Os argumentos mask_forced e mask_required são excludentes')
        validate_mask(mask)

        self.mask_required = mask_required
        self.mask_forced = mask_forced
        self.mask = mask
        self.validator = validate_mod11
        kwargs['max_length'] = len(mask)
        kwargs['min_length'] = len(mask) if mask_required or mask_forced else len(only_digits(mask))
        super(MaskField, self).__init__(*args, **kwargs)

    def clean(self, value):
        """
        O valor pode ou não ter máscara.

        :parameter value Valor a ser validado
        """
        try:
            if value in self.empty_values:
                return super(MaskField, self).clean(value)
            if self.mask_forced:
                value = super(MaskField, self).clean(apply_mask(value, self.mask))
            else:
                value = super(MaskField, self).clean(value)
            return self.clean_mask(value)
        except ValidationException as e:
            raise ValidationError(e.message)

    def clean_mask(self, value):
        if self.mask_required:
            result = validate_dv_by_mask(only_digits(value), self.mask, False, validate_dv=self.validator)
            if result != value:
                raise ValidationException('O campos tem que ser informado com máscara')
            return value
        else:
            return validate_dv_by_mask(value, self.mask, self.mask_forced, validate_dv=self.validator)

    def widget_attrs(self, widget):
        attrs = super(MaskField, self).widget_attrs(widget)
        attrs.update({'mask': str(self.mask)})
        return attrs


class CPFField(MaskField):
    """
    Um campo de formulário que valida o número do CPF.

    See: http://en.wikipedia.org/wiki/Cadastro_de_Pessoas_F%C3%ADsicas
    """
    def __init__(self, mask_forced=True, mask_required=False, *args, **kwargs):
        super(CPFField, self).__init__(CPF_MASK, mask_forced, mask_required, *args, **kwargs)


class CNPJField(MaskField):
    """
    Um campo de formulário que valida o número do CNPJ.

    See: http://en.wikipedia.org/wiki/National_identification_number#Brazil
    """
    def __init__(self, mask_forced=True, mask_required=False, *args, **kwargs):
        super(CNPJField, self).__init__(CNPJ_MASK, mask_forced, mask_required, *args, **kwargs)
        self.validator = validate_cnpj


class CEPField(MaskField):
    """
    Um campo de formulário que valida o número do CEP.
    """
    def __init__(self, mask_forced=True, mask_required=False, *args, **kwargs):
        super(CEPField, self).__init__(CEP_MASK, mask_forced, mask_required, *args, **kwargs)



import os
from collections import OrderedDict

from django.apps import apps
from django.contrib.staticfiles.finders import get_finders
from django.contrib.staticfiles.storage import staticfiles_storage
from django.core.files.storage import FileSystemStorage
from django.core.management.base import BaseCommand, CommandError
from django.core.management.color import no_style
from django.utils.functional import cached_property
from django_brfied.django_brfied.models import UnidadeFederativa, Municipio
from ...migrations import UNIDADE_FEDERATIVA_ROWS, MUNICIPIO_ROWS


class Command(BaseCommand):
    help = "Importa as UFs e os Municípios para a base"
    # requires_system_checks = False

    # def __init__(self, *args, **kwargs):
    #     super().__init__(*args, **kwargs)
    #     self.ignore_patterns = []
    #
    # def add_arguments(self, parser):
    #     parser.add_argument(
    #         '--no-default-ignore', action='store_false', dest='use_default_ignore_patterns',
    #         help="Don't ignore the common private glob-style patterns (defaults to 'CVS', '.*' and '*~').",
    #     )
    #
    # def set_options(self, **options):
    #     """
    #     Set instance variables based on an options dict
    #     """
    #     if options['use_default_ignore_patterns']:
    #         ignore_patterns += apps.get_app_config('staticfiles').ignore_patterns

    def handle(self, **options):
        print('Importando UF')
        for uf in UNIDADE_FEDERATIVA_ROWS:
            UnidadeFederativa.objects.\
                update_or_create(sigla=uf[0], defaults={'nome': uf[1], 'codigo': uf[2], 'regiao': uf[3]})
        print('UF importadas\n')

        print('Importando municípios')
        i = 1
        q = len(MUNICIPIO_ROWS)
        for m in MUNICIPIO_ROWS:
            if i%500 == 0:
                print('\tImportados %3.2f%%' % ((i / q) * 100))
            Municipio.objects.update_or_create(codigo=m[0], defaults={'nome': m[1], 'uf_id': m[2]})
            i += 1
        print('Municípios importados')



# Generated by Django 2.0.1 on 2018-01-08 16:03

from django.db import migrations, models
import django_brfied.django_brfied.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Municipio',
            fields=[
                ('codigo', models.CharField(max_length=6, primary_key=True, serialize=False, verbose_name='Código')),
                ('nome', models.CharField(max_length=255, verbose_name='Código')),
            ],
            options={
                'verbose_name': 'Município',
                'verbose_name_plural': 'Municípios',
                'ordering': ['nome'],
            },
        ),
        migrations.CreateModel(
            name='UnidadeFederativa',
            fields=[
                ('sigla', models.CharField(max_length=2, primary_key=True, serialize=False, verbose_name='Sigla')),
                ('codigo', models.CharField(max_length=2, unique=True, verbose_name='Código')),
                ('nome', models.CharField(max_length=250, verbose_name='Nome')),
                ('regiao', models.CharField(choices=[('N', 'Norte'), ('NE', 'Nordeste'), ('SE', 'Sudeste'), ('S', 'Sul'), ('CO', 'Centro-Oeste')], max_length=2, verbose_name='Região')),
            ],
            options={
                'verbose_name': 'Unidade federativa',
                'verbose_name_plural': 'Unidades federativas',
                'ordering': ['nome'],
            },
        ),
        migrations.AddField(
            model_name='municipio',
            name='uf',
            field=django_brfied.django_brfied.models.UFField(on_delete=None, to='django_brfied.UnidadeFederativa', verbose_name='UF'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
The MIT License (MIT)

Copyright 2015 Umbrella Tech.

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
from __future__ import unicode_literals
from django.core.exceptions import ValidationError
from django.db.models import Model, CharField
from django.db.models import ForeignKey as OriginalForeignKey, ManyToManyField as OriginalManyToManyField
from python_brfied import validate_dv_by_mask, validate_mask, validate_mod11, validate_cnpj
from python_brfied import only_digits, apply_mask, ValidationException
from python_brfied import CPF_MASK, CNPJ_MASK, CEP_MASK, RegiaoChoices
from python_brfied import SexoChoices
from django_brfied.django_brfied import forms
from django_brfied.django_brfied.validators import CPFValidator

# __all__ = ['MaskField', 'CPFField', 'CNPJField', 'CEPField', 'UFField', 'UnidadeFederativa', 'ForeignKey', 'Sexo']

__author__ = 'Kelson da Costa Medeiros <kelsoncm@gmail.com>'


class MaskField(CharField):
    description = "String with mask %(mask)"

    def __init__(self, verbose_name='', mask='', mask_stored=False, *args, **kwargs):
        self.mask, self.mask_stored = mask, mask_stored
        self.validator = validate_mod11
        kwargs['max_length'] = len(mask) if mask_stored else len(only_digits(mask))
        validate_mask(self.mask)
        super(MaskField, self).__init__(verbose_name, *args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(MaskField, self).deconstruct()
        if self.mask:
            kwargs['mask'] = self.mask
        if self.mask_stored:
            kwargs['mask_stored'] = self.mask_stored
        kwargs['max_length'] = len(self.mask) if self.mask_stored else len(only_digits(self.mask))
        return name, path, args, kwargs

    def from_db_value(self, value, *args, **kwargs):
        try:
            if value is None:
                return value
            return apply_mask(only_digits(value), self.mask)
        except ValidationException as e:
            raise ValidationError(e.message)

    def get_db_prep_value(self, value, connection, prepared=False):
        try:
            value = super(MaskField, self).get_db_prep_value(value, connection, prepared)
            if value is None:
                return None
            return apply_mask(only_digits(value), self.mask) if self.mask_stored else only_digits(value)
        except ValidationException as e:
            raise ValidationError(e.message)

    def validate(self, value, model_instance):
        super(MaskField, self).validate(value, model_instance)
        try:
            validate_dv_by_mask(value, self.mask, validate_dv=self.validator)
        except ValidationException as e:
            raise ValidationError(e.message)


class CPFField(MaskField):
    description = "CPF field"

    def __init__(self, max_length=11, validators=[CPFValidator()], *args, **kwargs):
        kwargs['mask'] = CPF_MASK
        kwargs['max_length'] = max_length
        kwargs['validators'] = validators
        super(CPFField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = kwargs.copy()
        defaults['form_class'] = forms.CPFField
        return super(CPFField, self).formfield(**defaults)


class CNPJField(MaskField):
    description = "CNPJ field"

    def __init__(self, *args, **kwargs):
        kwargs['mask'] = CNPJ_MASK
        super(CNPJField, self).__init__(*args, **kwargs)
        self.validator = validate_cnpj

    def formfield(self, **kwargs):
        defaults = kwargs.copy()
        defaults['form_class'] = forms.CNPJField
        return super(CNPJField, self).formfield(**defaults)


class CEPField(MaskField):
    description = "CEP field"

    def __init__(self, verbose_name='CEP', mask=CEP_MASK, mask_stored=True, *args, **kwargs):
        super(CEPField, self).__init__(verbose_name, mask, mask_stored, *args, **kwargs)

    def formfield(self, **kwargs):
        defaults = kwargs.copy()
        defaults['form_class'] = forms.CEPField
        return super(CEPField, self).formfield(**defaults)


class ForeignKey(OriginalForeignKey):

    def __init__(self, verbose_name, to, on_delete=None, related_name=None, related_query_name=None,
                 limit_choices_to=None, parent_link=False, to_field=None, db_constraint=True, **kwargs):
        kwargs['verbose_name'] = verbose_name
        super(ForeignKey, self).__init__(to, on_delete, related_name, related_query_name, limit_choices_to, parent_link,
                                         to_field, db_constraint, **kwargs)


class ManyToManyField(OriginalManyToManyField):

    def __init__(self, verbose_name, to, related_name=None, related_query_name=None, limit_choices_to=None,
                 symmetrical=None, through=None, through_fields=None, db_constraint=True, db_table=None, swappable=True,
                 **kwargs):
        super(ManyToManyField, self).__init__(to, related_name, related_query_name, limit_choices_to, symmetrical,
                                              through, through_fields, db_constraint, db_table, swappable,
                                              verbose_name=verbose_name, **kwargs)


class SexoField(CharField):
    description = "Sex"

    def __init__(self, verbose_name='Sexo', max_length=1, choices=SexoChoices.CHOICES, *args, **kwargs):
        super(SexoField, self).__init__(verbose_name=verbose_name, max_length=max_length, choices=choices,
                                        *args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(SexoField, self).deconstruct()
        kwargs['max_length'] = 1
        kwargs['choices'] = SexoChoices.CHOICES
        return name, path, args, kwargs


class URL(CharField):
    description = "Sex"

    def __init__(self, verbose_name='Sexo', max_length=1, choices=SexoChoices.CHOICES, *args, **kwargs):
        super(SexoField, self).__init__(verbose_name=verbose_name, max_length=max_length, choices=choices,
                                        *args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(SexoField, self).deconstruct()
        kwargs['max_length'] = 1
        kwargs['choices'] = SexoChoices.CHOICES
        return name, path, args, kwargs


class UnidadeFederativa(Model):
    sigla = CharField('Sigla', max_length=2, primary_key=True)
    codigo = CharField('Código', max_length=2, unique=True)
    nome = CharField('Nome', max_length=250)
    regiao = CharField('Região', max_length=2, choices=RegiaoChoices.CHOICES)

    class Meta:
        verbose_name = 'Unidade federativa'
        verbose_name_plural = 'Unidades federativas'
        ordering = ['nome']

    def __str__(self):
        return '%s (%s)' % (self.nome, self.sigla, )


class UFField(ForeignKey):
    def __init__(self, verbose_name='UF', to=UnidadeFederativa, on_delete=None, related_name=None,
                 related_query_name=None, limit_choices_to=None, parent_link=False, to_field=None,
                 db_constraint=True, **kwargs):
        super(UFField, self).__init__(verbose_name, to, on_delete, related_name, related_query_name, limit_choices_to,
                                      parent_link, to_field, db_constraint, **kwargs)


class Municipio(Model):
    codigo = CharField('Código', max_length=6, primary_key=True)
    nome = CharField('Código', max_length=255)
    uf = UFField()

    class Meta:
        verbose_name = 'Município'
        verbose_name_plural = 'Municípios'
        ordering = ['nome']

    def __str__(self):
        return '%s/%s' % (self.nome, self.uf_id, )


class MunicipioField(ForeignKey):
    def __init__(self, verbose_name='Município', to=Municipio, on_delete=None, related_name=None,
                 related_query_name=None, limit_choices_to=None, parent_link=False, to_field=None,
                 db_constraint=True, **kwargs):
        super(MunicipioField, self).__init__(verbose_name, to, on_delete, related_name, related_query_name,
                                             limit_choices_to, parent_link, to_field, db_constraint, **kwargs)



# -*- coding: utf-8 -*-
from distutils.core import setup
setup(
    name='django_brfied',
    packages=['django_brfied', 'django_brfied/migrations', 'django_brfied/management/commands', ],
    package_dir={'django_brfied': 'django_brfied'},
    # package_data={'django_brfied': ['django_brfied/static/js/*', 'django_brfied/*', 'django_brfied/migrations/*', 'django_brfied/management/commands/*'],},
    version='0.5.1',
    download_url='https://github.com/kelsoncm/django_brfied/releases/tag/0.5.1',
    description='Django Application specific brazilian fields types',
    author='Kelson da Costa Medeiros',
    author_email='kelsoncm@gmail.com',
    url='https://github.com/kelsoncm/django_brfied',
    keywords=['django', 'BR', 'Brazil', 'Brasil', 'model', 'form', 'locale', ],
    classifiers=[],
    install_requires=['python_brfied', ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.deconstruct import deconstructible
from python_brfied import validate_cpf


@deconstructible
class CPFValidator(object):
    message = 'Informe um email válido'
    code = 'invalid'

    def __init__(self, message=None, code=None):
        if message is not None:
            self.message = message
        if code is not None:
            self.code = code

    def __call__(self, value):
        return validate_cpf(value)

    def __eq__(self, other):
        return (
            isinstance(other, CPFValidator) and
            (self.message == other.message) and
            (self.code == other.code)
        )


class CPFAuthValidator:
    def validate(self, password, user=None):
        validate_cpf(user.username)

    def get_help_text(self):
        return 'Informe um CPF válido'
